-- Class and talent informations
local TM_TalentTemplateMenue = nil;
local TM_CurrentTalents = nil;
local TM_TalentTemplatePreview = nil;
local TM_AddTaltentsFrame = nil;
local TM_TalentTemplatePreviewContent = nil;
local TM_TalentTemplatePreviewActivate = nil;
-- Create the saved value if it not exist

if TM_Talents == nil then
	TM_Talents = {}
end

 
-- Create talent template drop down menu
function TM_TaltenTemplate_Initial(self,level,menuList)
	if level == 1 then
		-- create the headline
		local info = UIDropDownMenu_CreateInfo();
		info.text = "Talent Master";
		info.isTitle = 1;
		UIDropDownMenu_AddButton(info);
		-- create the add button
		info = UIDropDownMenu_CreateInfo();
		info.text = "Add Current Talents";
		info.func = function() TM_AddTaltentsFrame:Show(); self.text = info.text; end;
		UIDropDownMenu_AddButton(info);
		-- create seperator
		info = UIDropDownMenu_CreateInfo();
		info.disabled  = true;
		UIDropDownMenu_AddButton(info);
		-- load data
		for i in pairs(TM_Talents) do
			info = UIDropDownMenu_CreateInfo();
			info.text = TM_Talents[i]["name"];
			info.func = function() TM_LoadPreview(TM_Talents[i]); end;
			UIDropDownMenu_AddButton(info);
		end
	end
end

-- function to get the talentpoints
function TM_GetTalentPointsByTab(tabIndex)
	local numTalents = GetNumTalents(tabIndex);
	local talentPoints = 0;
	
	for i = 1, numTalents, 1 do
		name, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(tabIndex, i);
		talentPoints = talentPoints + currentRank;
	end
	
	return talentPoints;
end

-- function to get the current talents from a tabIndex
function TM_GetTalentsByTab(tabIndex)
	local _talents = {};
	local numTalents = GetNumTalents(tabIndex);
	
	for i = 1, numTalents, 1 do
		name, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(tabIndex, i);
		_talents[i] = { name = name , iconPath = iconPath, tier = tier, column = column, currentRank = currentRank, maxRank = maxRank, isExceptional = isExceptional, meetsPrereq = meetsPrereq };
	end
	
	return _talents;
end

-- function to get the current talents as table
function TM_GetCurrentTalents() 
	local _talents = {};
	local numTabs = GetNumTalentTabs();
	
	for i = 1, numTabs, 1 do
		_talents[i] = TM_GetTalentsByTab(i);
	end
	
	return _talents;
end

-- Returns the skillpoints from one talent build specc
function TM_GetSkillPointsByTable(tab) 
	local skillPoints = 0;
	
	for i in pairs(tab) do
		skillPoints = skillPoints + tab[i]["currentRank"];
	end
	
	return skillPoints;
end

-- Set the actual used talen build
function TM_SetSelectedTemplate()
	UIDropDownMenu_SetText(TM_TalentTemplateMenue, TM_SelectedTalentTemplate["name"]);
end

-- Create the preview for a talent template
function TM_LoadPreview(template)
	TM_TalentTemplatePreview:Show();
	local content = "<html><body>";
	local content = content .. "<h1>".. template["name"] .. "</h1><br/>";
	
	for iTabs in pairs(template["talents"]) do
		for iTalent in pairs(template["talents"][iTabs]) do
			if template["talents"][iTabs][iTalent]["currentRank"] ~= 0 then
				content = content .. "<p>".. "|T"..template["talents"][iTabs][iTalent]["iconPath"]..":0|t".." [|cff00ff00".. template["talents"][iTabs][iTalent]["currentRank"].."|r] |cffffff00" .. template["talents"][iTabs][iTalent]["name"] .."|r</p>";
			end
		end
	end
	local content = content .. "</body></html>";
	
	TM_TalentTemplatePreviewContent:SetText(content);
	TM_TalentTemplatePreviewActivate:SetScript("OnClick", function() TM_ActivateTemplate(template); TM_TalentTemplatePreview:Hide(); end);
end

-- Starts to activate the template
function TM_ActivateTemplate(template)
	local spellList = { currentSpell = 1 , spells = {}};
	for iTabs in pairs(template["talents"]) do
		for iTalent in pairs(template["talents"][iTabs]) do 
			for iRank = 1, template["talents"][iTabs][iTalent]["currentRank"], 1 do
				table.insert(spellList["spells"], {tab = iTabs, index = iTalent, name = template["talents"][iTabs][iTalent]["name"], icon = template["talents"][iTabs][iTalent]["iconPath"] });
			end			
		end		
	end
	
	TM_SelectedTalentTemplate = template;
	print("|cffff0000[Talent Master] : |r", "|cffffff00" .. "You learn the talents asynchrone, this process will take arround ", #spellList["spells"],  "seconds. You can containue playing while learning the talents. |r");
	C_Timer.After(1, function() TM_LearnSpellAsync(spellList) end);
end

-- Learns a spell from the spellList Async
function TM_LearnSpellAsync(spellList)
	local current = spellList["currentSpell"];
	local nextSpell = current + 1;
	LearnTalent(spellList["spells"][current]["tab"], spellList["spells"][current]["index"]);
	print("|cffff0000[Talent Master] Spell Learned: |r", "|T"..spellList["spells"][current]["icon"]..":0|t", "|cffffff00" .. spellList["spells"][current]["name"] .. "|r");
	spellList["currentSpell"] = nextSpell;
	
	if nextSpell == #spellList["spells"] + 1 then
		print("|cffff0000[Talent Master] : |r", "|cffffff00" .. "DONE!" .. "|r");
	else
		C_Timer.After(1, function() TM_LearnSpellAsync(spellList) end);
	end
end

-- Adds the current talets to the talent template and activite them
function TM_AddCurrentTalents()
	local talents = TM_GetCurrentTalents();
	local requiredSkillpoints = 0;
	local name = "(";
	for i in pairs(talents) do
		local skillPoints = TM_GetSkillPointsByTable(talents[i]);
		requiredSkillpoints = requiredSkillpoints + skillPoints;
		if i ~= #talents then
			name = name .. skillPoints .. "/";
		else
			name = name .. skillPoints;
		end
	end
	name = name .. ")" .. " " .. TM_AddTalentsEditBox:GetText();
	local talenTemplate = { name = name , talents = talents , requiredSkillpoints = requiredSkillpoints};
	
	table.insert(TM_Talents, talenTemplate);
	TM_SelectedTalentTemplate = talenTemplate;
	TM_AddTaltentsFrame:Hide();
end


-- Adds the frames to the talent frame
function TM_TalentFrame_LoadUI()
	-- Create the dropdown menu
	if TM_TalentTemplateMenue == nil then
		TM_TalentTemplateMenue = CreateFrame("Frame", "TM_TalentTemplateMenue", TalentFrame, "UIDropDownMenuTemplate");
		UIDropDownMenu_SetWidth(TM_TalentTemplateMenue, 250)
		TM_TalentTemplateMenue:SetPoint("TOPLEFT", 55, -40);
		UIDropDownMenu_Initialize(TM_TalentTemplateMenue, TM_TaltenTemplate_Initial);
	else
		UIDropDownMenu_Initialize(TM_TalentTemplateMenue, TM_TaltenTemplate_Initial);
	end
		
	-- prints the name from the selected template
	if TM_SelectedTalentTemplate ~= nil then
		TM_SetSelectedTemplate();
	end
	
	-- Create the add window
	if TM_AddTaltentsFrame == nil then
		TM_AddTaltentsFrame = CreateFrame("Frame", "TM_AddTaltentsFrame", TalentFrame, "TranslucentFrameTemplate");
		TM_AddTaltentsFrame:SetPoint("TOP", -20, -85);
		TM_AddTaltentsFrame:SetSize(300, 75);
		TM_AddTaltentsFrame:SetFrameStrata("HIGH");
		TM_AddTaltentsFrame:Hide();
	
		local TM_AddTalentsEditBox = CreateFrame("EditBox", "TM_AddTalentsEditBox", TM_AddTaltentsFrame, "InputBoxTemplate");
		TM_AddTalentsEditBox:SetPoint("CENTER", 0, 10);
		TM_AddTalentsEditBox:SetSize(250, 30);
		TM_AddTalentsEditBox:SetAutoFocus(false);

		local TM_CloseAddTalentsWindowButton = CreateFrame("Button", "TM_CloseAddTalentsWindowButton", TM_AddTaltentsFrame, "UIPanelButtonTemplate");
		TM_CloseAddTalentsWindowButton:SetText("X");
		TM_CloseAddTalentsWindowButton:SetSize(20, 20);
		TM_CloseAddTalentsWindowButton:SetPoint("TOPRIGHT", -5, -5);
		TM_CloseAddTalentsWindowButton:SetScript("OnClick", function() TM_AddTaltentsFrame:Hide() end);

		local TM_AddTalentsButton = CreateFrame("Button", "TM_AddTalentsButton", TM_AddTaltentsFrame, "UIPanelButtonTemplate");
		TM_AddTalentsButton:SetText("Add Talents");
		TM_AddTalentsButton:SetSize(100, 20);
		TM_AddTalentsButton:SetPoint("BOTTOM", 0, 15);
		TM_AddTalentsButton:SetScript("OnClick", function() TM_AddCurrentTalents() end);
	end
	
	if TM_TalentTemplatePreview == nil then
		TM_TalentTemplatePreview = CreateFrame("Frame", "TM_TalentTemplatePreview", TalentFrame, "TooltipBorderedFrameTemplate");
		TM_TalentTemplatePreview:SetPoint("TOPRIGHT", 200, -10, "TOPLEFT");
		TM_TalentTemplatePreview:SetSize(200, (TalentFrame:GetHeight() - 85));
		TM_TalentTemplatePreview:Hide();
		
		local TM_TalentTemplatePreviewScrollFrame = CreateFrame("ScrollFrame", "TM_TalentTemplatePreviewScrollFrame", TM_TalentTemplatePreview, "UIPanelScrollFrameTemplate");
		TM_TalentTemplatePreviewScrollFrame:SetPoint("TOPLEFT", 20, -30);
		TM_TalentTemplatePreviewScrollFrame:SetSize("150", "350");

		local TM_TalentTemplatePreviewScrollChild = CreateFrame("Frame", "LFG_Changelog_ScrollChild", nil);
		TM_TalentTemplatePreviewScrollChild:SetSize("150", "800");
		TM_TalentTemplatePreviewScrollChild:SetPoint("TOPLEFT");
		TM_TalentTemplatePreviewScrollFrame:SetScrollChild(TM_TalentTemplatePreviewScrollChild);

		TM_TalentTemplatePreviewContent = CreateFrame('SimpleHTML', 'LFG_Changelog_SimpleHTML', TM_TalentTemplatePreviewScrollChild);
		TM_TalentTemplatePreviewContent:SetPoint("TOPLEFT");
		TM_TalentTemplatePreviewContent:SetFont('Fonts\\FRIZQT__.TTF', 11);
		TM_TalentTemplatePreviewContent:SetFont('h1', 'Fonts\\FRIZQT__.TTF', 12);
		TM_TalentTemplatePreviewContent:SetFont('h2', 'Fonts\\FRIZQT__.TTF', 10);
		TM_TalentTemplatePreviewContent:SetSize(450, 800);
		TM_TalentTemplatePreviewContent:SetHyperlinkFormat("|cff71C671|H%s|h[%s]|h|r");
	
		TM_TalentTemplatePreviewActivate = CreateFrame("Button", "TM_AddTalentsButton", TM_TalentTemplatePreview, "UIPanelButtonTemplate");
		TM_TalentTemplatePreviewActivate:SetText("Activate");
		TM_TalentTemplatePreviewActivate:SetSize(100, 20);
		TM_TalentTemplatePreviewActivate:SetPoint("BOTTOMLEFT", 15, 15);
	else
		TM_TalentTemplatePreview:Hide();
	end
	
end
hooksecurefunc("TalentFrame_LoadUI", TM_TalentFrame_LoadUI);
